==> fap-0.9.0pre1.tar.bz2: fap-0.9.0pre1/src/audio/input/mpg123Backend.cpp <==
/*
 *  Copyright (C) 2009-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "mpg123Backend.h"

#ifdef HAVE_MPG123_H

#include "settings.h"
#include "genres.h"
#include "loadsym.h"

// mp3|mp2
#define EXT	"*.mp3"

#define CREDITS		"MPEG Audio Decoder library\ncopyright 1995-2008 by the mpg123 project"
#define LINK		"http://mpg123.org/"

const char mpg123Backend::name[]="Mpg123";

#ifdef _WIN32
# define MPG123LIB	"libmpg123-0.dll"
#else
# define MPG123LIB	"libmpg123.so"
#endif

int mpg123Backend::_status=0;
const AutoDLL mpg123Backend::_dll(MPG123LIB);

int (*mpg123Backend::dl_mpg123_init)(void)=0;
void (*mpg123Backend::dl_mpg123_exit)(void)=0;
mpg123_handle* (*mpg123Backend::dl_mpg123_new)(const char*, int*)=0;
void (*mpg123Backend::dl_mpg123_delete)(mpg123_handle*)=0;
const char* (*mpg123Backend::dl_mpg123_plain_strerror)(int)=0;
int (*mpg123Backend::dl_mpg123_open_fd)(mpg123_handle*, int)=0;
int (*mpg123Backend::dl_mpg123_close)(mpg123_handle*)=0;
int (*mpg123Backend::dl_mpg123_scan)(mpg123_handle*)=0;
int (*mpg123Backend::dl_mpg123_getformat)(mpg123_handle*, long*, int*, int*)=0;
off_t (*mpg123Backend::dl_mpg123_length)(mpg123_handle*)=0;
int (*mpg123Backend::dl_mpg123_read)(mpg123_handle*, unsigned char*, size_t, size_t*)=0;
off_t (*mpg123Backend::dl_mpg123_seek)(mpg123_handle*, off_t, int)=0;
int (*mpg123Backend::dl_mpg123_id3)(mpg123_handle*, mpg123_id3v1**, mpg123_id3v2**)=0;
const char** (*mpg123Backend::dl_mpg123_supported_decoders)(void)=0;
int(*mpg123Backend::dl_mpg123_replace_reader)(mpg123_handle*, ssize_t(*)(int, void *, size_t), off_t(*)(int, off_t, int))=0;
int (*mpg123Backend::dl_mpg123_param)(mpg123_handle*, enum mpg123_parms, long, double)=0;

FXString mpg123Backend::_decoders=FXString::null;

FXFile mpg123Backend::_file[2];
int mpg123Backend::_count=0;

mpg123Config_t mpg123Backend::_settings;

/*****************************************************************/

size_t mpg123Backend::fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds) {

	size_t n;
	const int err=dl_mpg123_read(_handle, (unsigned char*)buffer, bufferSize, &n);
	if (err!=MPG123_OK)
		return 0;

	return n;
}

/*****************************************************************/

bool mpg123Backend::init() {

	if (!_dll.loaded())
		return false;

	LOADSYM(_dll, mpg123_init, int(*)(void))
	LOADSYM(_dll, mpg123_exit, void(*)(void))
	LOADSYM(_dll, mpg123_new, mpg123_handle*(*)(const char*, int*))
	LOADSYM(_dll, mpg123_delete, void(*)(mpg123_handle*))
	LOADSYM(_dll, mpg123_plain_strerror, const char*(*)(int))
	LOADSYMSFX(_dll, mpg123_open_fd, int(*)(mpg123_handle*, int))
	LOADSYM(_dll, mpg123_close, int(*)(mpg123_handle*))
	LOADSYM(_dll, mpg123_scan, int(*)(mpg123_handle*))
	LOADSYM(_dll, mpg123_getformat, int(*)(mpg123_handle*, long*, int*, int*))
	LOADSYMSFX(_dll, mpg123_length, off_t(*)(mpg123_handle*))
	LOADSYM(_dll, mpg123_read, int(*)(mpg123_handle*, unsigned char*, size_t, size_t*))
	LOADSYMSFX(_dll, mpg123_seek, off_t(*)(mpg123_handle*, off_t, int))
	LOADSYM(_dll, mpg123_id3, int(*)(mpg123_handle*, mpg123_id3v1**, mpg123_id3v2**))
	LOADSYM(_dll, mpg123_supported_decoders, const char**(*)(void))
	TRYLOADSYM(_dll, mpg123_replace_reader, mpg123_replace_reader_64, int(*)(mpg123_handle*, ssize_t(*)(int, void *, size_t), off_t(*)(int, off_t, int)))
	LOADSYM(_dll, mpg123_param, int(*)(mpg123_handle*, enum mpg123_parms, long, double))

	return true;
}

bool mpg123Backend::supports(const char* fileName) {

#if FOX_MINOR>6
	return FXPath::match(FXPath::name(fileName), EXT, NoEscape|PathName|CaseFold);
#else
	return FXPath::match(FXString(EXT), FXPath::name(fileName), FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD);
#endif
}

inline FXString mpg123Backend::ext() const { return FXString(EXT); }

mpg123Backend::mpg123Backend() :
	inputBackend(name),
	_handle(0) {

	loadSettings();

	if (++_status>1)
		return;

	const int err=dl_mpg123_init();
	if (err==MPG123_OK) {
		_decoders="auto";
		const char** decoders=dl_mpg123_supported_decoders();
		while (*decoders) {
			FXTRACE((10, "Decoder: %s\n", *decoders));
			_decoders.append('\n');
			_decoders.append(*decoders);
			++decoders;
		}
	} else {
		fxmessage(dl_mpg123_plain_strerror(err));
		_status=0;
	}
}

mpg123Backend::~mpg123Backend() {

	close();

	if (--_status==0)
		dl_mpg123_exit();
}

void mpg123Backend::loadSettings() {

	_settings.fastscan=load("Fastscan", TRUE);
	_settings.decoder=load("Decoder", "auto");
}

void mpg123Backend::saveSettings() {

	save("Fastscan", _settings.fastscan);
	save("Decoder", _settings.decoder);
}

bool mpg123Backend::open(const FXString& fileName) {

	if (!_status)
		return false;

	close();

	_fd=_count&1;
	if (!_file[_fd].open(fileName))
		return false;

	int err;
	FXTRACE((10, "Setting decoder: %s\n", compare(_settings.decoder, "auto")?_settings.decoder.text():0));
	_handle=dl_mpg123_new(compare(_settings.decoder, "auto")?_settings.decoder.text():0, &err);
	if (!_handle) {
		fxmessage(dl_mpg123_plain_strerror(err));
		return false;
	}

	err=dl_mpg123_replace_reader(_handle, read_func, seek_func);
	if (err==MPG123_OK) {
		err=dl_mpg123_open_fd(_handle, _fd);
	}
	if (err!=MPG123_OK) {
		dl_mpg123_delete(_handle);
		fxmessage(dl_mpg123_plain_strerror(err));
		return false;
	}

	if (!_settings.fastscan) {
		err=dl_mpg123_scan(_handle);
		if (err!=MPG123_OK) {
			fxmessage(dl_mpg123_plain_strerror(err));
		}
	}

	int encoding;
	err=dl_mpg123_getformat(_handle, &_samplerate, &_channels, &encoding);

	err=dl_mpg123_length(_handle);
	time(err/_samplerate);

	err=dl_mpg123_param(_handle, MPG123_RVA,
		SETTINGS->replayGain()?(SETTINGS->replayGainMode()?MPG123_RVA_ALBUM:MPG123_RVA_MIX):MPG123_RVA_OFF,
		0.);

	mpg123_id3v1* id3v1;
	mpg123_id3v2* id3v2;
	err=dl_mpg123_id3(_handle, &id3v1, &id3v2);
	if (err==MPG123_OK) {
		FXString info;

		if (id3v2 && id3v2->title)
			info=FXString(id3v2->title->p, id3v2->title->fill);
		else if (id3v1)
			info=FXString(id3v1->title, FXMIN(strlen(id3v1->title), 30)).trim();
		else
			info=FXString::null;
		FXTRACE((10, "TITLE: %s\n", info.text()));
		_metaData.addInfo(metaData::TITLE, info.text());

		if (id3v2 && id3v2->artist)
			info=FXString(id3v2->artist->p, id3v2->artist->fill);
		else if (id3v1)
			info=FXString(id3v1->artist, FXMIN(strlen(id3v1->artist), 30)).trim();
		else
			info=FXString::null;
		FXTRACE((10, "ARTIST: %s\n", info.text()));
		_metaData.addInfo(metaData::ARTIST, info.text());

		if (id3v2 && id3v2->album)
			info=FXString(id3v2->album->p, id3v2->album->fill);
		else if (id3v1)
			info=FXString(id3v1->album, FXMIN(strlen(id3v1->album), 30)).trim();
		else
			info=FXString::null;
		FXTRACE((10, "ALBUM: %s\n", info.text()));
		_metaData.addInfo(metaData::ALBUM, info.text());

		if (id3v2 && id3v2->genre) {
			info=FXString(id3v2->genre->p, id3v2->genre->fill);
			if (info.find('(')>=0) {
				const FXuint idx=FXUIntVal(info.after('(').before(')'));
				if (idx<GENRES)
					info=FXString(genre[idx]);
			}
		} else if (id3v1 && id3v1->genre<GENRES)
			info=genre[id3v1->genre];
		else
			info=FXString::null;
		FXTRACE((10, "GENRE: %s\n", info.text()));
		_metaData.addInfo(metaData::GENRE, info.text());

		if (id3v2 && id3v2->year)
			info=FXString(id3v2->year->p, id3v2->year->fill);
		else if (id3v1)
			info=FXString(id3v1->year, FXMIN(strlen(id3v1->year), 4)).trim();
		else
			info=FXString::null;
		FXTRACE((10, "YEAR: %s\n", info.text()));
		_metaData.addInfo(metaData::YEAR, info.text());

		if (id3v1 && id3v1->comment[28]==0)
#if FOX_MINOR>6
			info=FXString::value(id3v1->comment[29]);
#else
			info=FXStringVal(id3v1->comment[29]);
#endif
		else
			info=FXString::null;
		FXTRACE((10, "TRACK: %s\n", info.text()));
		_metaData.addInfo(metaData::TRACK, info.text());

		info=FXString::null;
		if (id3v2) {
			for (unsigned int i=0; i<id3v2->comments; i++) {
				mpg123_text *entry = &id3v2->comment_list[i];
				if (entry->description.fill==0 || entry->description.p[0]==0)
					info=FXString(entry->text.p, entry->text.fill).trim();
			}
		}

		if (info.empty() && id3v1)
			info=FXString(id3v1->comment, FXMIN(strlen(id3v1->comment), 30)).trim();

		if (!info.empty()) {
			FXTRACE((10, "COMMENT: %s\n", info.text()));
			_metaData.addInfo(metaData::COMMENT, info.text());
		}
	}

	_count++;

	songLoaded(fileName.text());
	return true;
}

void mpg123Backend::close() {

	if (!songLoaded())
		return;

	dl_mpg123_close(_handle);
	dl_mpg123_delete(_handle);

	_file[_fd].close();

	songLoaded(0);
}

const bool mpg123Backend::rewind() {

	if (songLoaded()) {
		dl_mpg123_seek(_handle, 0, SEEK_SET);
		return true;
	}

	return false;
}

ssize_t mpg123Backend::read_func(int fd, void* ptr, size_t size) {

	return _file[fd].readBlock((char*)ptr, size);
}

off_t mpg123Backend::seek_func(int fd, off_t offset, int whence) {

	FXuint from;
	switch (whence) {
	case SEEK_SET:
		from=FXIO::Begin;
		break;
	case SEEK_CUR:
		from=FXIO::Current;
		break;
	case SEEK_END:
		from=FXIO::End;
		break;
	default:
		return -1;
	}
	return _file[fd].position(offset, from);
}

/*****************************************************************/

#define MPGSETTINGS	mpg123Backend::_settings

FXDEFMAP(mpg123Config) mpg123ConfigMap[] = {
	// Message_Type		ID				Message_Handler
	FXMAPFUNC(SEL_COMMAND,	mpg123Config::ID_DECODER,		mpg123Config::onCmdDecoder),
};

FXIMPLEMENT(mpg123Config, configFrame, mpg123ConfigMap, ARRAYNUMBER(mpg123ConfigMap))

mpg123Config::mpg123Config(FXPacker* win) :
	configFrame(win, mpg123Backend::name, CREDITS, LINK) {

	_fastscan.connect(MPGSETTINGS.fastscan);

	new FXLabel(matrix(), tr("Decoder"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
	FXListBox *decBox=new FXListBox(matrix(), this, mpg123Config::ID_DECODER, LAYOUT_FILL|FRAME_NORMAL);
	decBox->fillItems(mpg123Backend::_decoders);
	const FXint numItems=decBox->getNumItems();
	decBox->setNumVisible(FXMIN(numItems, 5));
	const FXint curItem=decBox->findItem(MPGSETTINGS.decoder);
	if (curItem>=0)
		decBox->setCurrentItem(curItem);

	new FXVerticalSeparator(extraLeft());

	FXVerticalFrame *vert=new FXVerticalFrame(extraLeft());
	new FXCheckButton(vert, tr("Fast scan\tScan only few frame of file, time detection may be inaccurate"),
		&_fastscan, FXDataTarget::ID_VALUE);
}

long mpg123Config::onCmdDecoder(FXObject*, FXSelector, void* ptr) {

	const FXint val=(FXint)(FXival)ptr;
	MPGSETTINGS.decoder=mpg123Backend::_decoders.section('\n', val);
	return 1;
}


#endif

==> fap-0.9.2.tar.bz2: fap-0.9.2/src/audio/input/mpg123Backend.cpp <==
/*
 *  Copyright (C) 2009-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "mpg123Backend.h"

#ifdef HAVE_MPG123_H

#include "settings.h"
#include "genres.h"
#include "loadsym.h"

// mp3|mp2
#define EXT	"*.mp3"

#define CREDITS		"MPEG Audio Decoder library\ncopyright 1995-2008 by the mpg123 project"
#define LINK		"http://mpg123.org/"

const char mpg123Backend::name[]="Mpg123";

#ifdef _WIN32
# define MPG123LIB	"libmpg123-0.dll"
#else
# define MPG123LIB	"libmpg123.so"
#endif

int mpg123Backend::_status=0;
const AutoDLL mpg123Backend::_dll(MPG123LIB);

int (*mpg123Backend::dl_mpg123_init)(void)=0;
void (*mpg123Backend::dl_mpg123_exit)(void)=0;
mpg123_handle* (*mpg123Backend::dl_mpg123_new)(const char*, int*)=0;
void (*mpg123Backend::dl_mpg123_delete)(mpg123_handle*)=0;
const char* (*mpg123Backend::dl_mpg123_plain_strerror)(int)=0;
int (*mpg123Backend::dl_mpg123_open_fd)(mpg123_handle*, int)=0;
int (*mpg123Backend::dl_mpg123_close)(mpg123_handle*)=0;
int (*mpg123Backend::dl_mpg123_scan)(mpg123_handle*)=0;
int (*mpg123Backend::dl_mpg123_getformat)(mpg123_handle*, long*, int*, int*)=0;
off_t (*mpg123Backend::dl_mpg123_length)(mpg123_handle*)=0;
int (*mpg123Backend::dl_mpg123_read)(mpg123_handle*, unsigned char*, size_t, size_t*)=0;
off_t (*mpg123Backend::dl_mpg123_seek)(mpg123_handle*, off_t, int)=0;
int (*mpg123Backend::dl_mpg123_id3)(mpg123_handle*, mpg123_id3v1**, mpg123_id3v2**)=0;
const char** (*mpg123Backend::dl_mpg123_supported_decoders)(void)=0;
int(*mpg123Backend::dl_mpg123_replace_reader)(mpg123_handle*, ssize_t(*)(int, void *, size_t), off_t(*)(int, off_t, int))=0;
int (*mpg123Backend::dl_mpg123_param)(mpg123_handle*, enum mpg123_parms, long, double)=0;

FXString mpg123Backend::_decoders=FXString::null;

FXFile mpg123Backend::_file[2];
int mpg123Backend::_count=0;

mpg123Config_t mpg123Backend::_settings;

/*****************************************************************/

size_t mpg123Backend::fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds) {

	size_t n;
	const int err=dl_mpg123_read(_handle, (unsigned char*)buffer, bufferSize, &n);
	if (err!=MPG123_OK)
		return 0;

	return n;
}

/*****************************************************************/

bool mpg123Backend::init() {

	if (!_dll.loaded())
		return false;

	LOADSYM(_dll, mpg123_init, int(*)(void))
	LOADSYM(_dll, mpg123_exit, void(*)(void))
	LOADSYM(_dll, mpg123_new, mpg123_handle*(*)(const char*, int*))
	LOADSYM(_dll, mpg123_delete, void(*)(mpg123_handle*))
	LOADSYM(_dll, mpg123_plain_strerror, const char*(*)(int))
	LOADSYMSFX(_dll, mpg123_open_fd, int(*)(mpg123_handle*, int))
	LOADSYM(_dll, mpg123_close, int(*)(mpg123_handle*))
	LOADSYM(_dll, mpg123_scan, int(*)(mpg123_handle*))
	LOADSYM(_dll, mpg123_getformat, int(*)(mpg123_handle*, long*, int*, int*))
	LOADSYMSFX(_dll, mpg123_length, off_t(*)(mpg123_handle*))
	LOADSYM(_dll, mpg123_read, int(*)(mpg123_handle*, unsigned char*, size_t, size_t*))
	LOADSYMSFX(_dll, mpg123_seek, off_t(*)(mpg123_handle*, off_t, int))
	LOADSYM(_dll, mpg123_id3, int(*)(mpg123_handle*, mpg123_id3v1**, mpg123_id3v2**))
	LOADSYM(_dll, mpg123_supported_decoders, const char**(*)(void))
	TRYLOADSYM(_dll, mpg123_replace_reader, mpg123_replace_reader_64, int(*)(mpg123_handle*, ssize_t(*)(int, void *, size_t), off_t(*)(int, off_t, int)))
	LOADSYM(_dll, mpg123_param, int(*)(mpg123_handle*, enum mpg123_parms, long, double))

	return true;
}

bool mpg123Backend::supports(const char* fileName) {

#if FOX_MINOR>6
	return FXPath::match(FXPath::name(fileName), EXT, NoEscape|PathName|CaseFold);
#else
	return FXPath::match(FXString(EXT), FXPath::name(fileName), FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD);
#endif
}

inline FXString mpg123Backend::ext() const { return FXString(EXT); }

mpg123Backend::mpg123Backend() :
	inputBackend(name),
	_handle(0) {

	loadSettings();

	if (++_status>1)
		return;

	const int err=dl_mpg123_init();
	if (err==MPG123_OK) {
		_decoders="auto";
		const char** decoders=dl_mpg123_supported_decoders();
		while (*decoders) {
			FXTRACE((10, "Decoder: %s\n", *decoders));
			_decoders.append('\n');
			_decoders.append(*decoders);
			++decoders;
		}
	} else {
		fxmessage(dl_mpg123_plain_strerror(err));
		_status=0;
	}
}

mpg123Backend::~mpg123Backend() {

	close();

	if (--_status==0)
		dl_mpg123_exit();
}

void mpg123Backend::loadSettings() {

	_settings.fastscan=load("Fastscan", TRUE);
	_settings.decoder=load("Decoder", "auto");
}

void mpg123Backend::saveSettings() {

	save("Fastscan", _settings.fastscan);
	save("Decoder", _settings.decoder);
}

bool mpg123Backend::open(const FXString& fileName) {

	if (!_status)
		return false;

	close();

	_fd=_count&1;
	if (!_file[_fd].open(fileName))
		return false;

	int err;
	FXTRACE((10, "Setting decoder: %s\n", compare(_settings.decoder, "auto")?_settings.decoder.text():0));
	_handle=dl_mpg123_new(compare(_settings.decoder, "auto")?_settings.decoder.text():0, &err);
	if (!_handle)
		goto error;

	err=dl_mpg123_replace_reader(_handle, read_func, seek_func);
	if (err==MPG123_OK) {
		err=dl_mpg123_open_fd(_handle, _fd);
	}
	if (err!=MPG123_OK) {
		dl_mpg123_delete(_handle);
		goto error;
	}

	if (!_settings.fastscan) {
		err=dl_mpg123_scan(_handle);
		if (err!=MPG123_OK) {
			fxmessage(dl_mpg123_plain_strerror(err));
		}
	}

	int encoding;
	err=dl_mpg123_getformat(_handle, &_samplerate, &_channels, &encoding);

	err=dl_mpg123_length(_handle);
	time(err/_samplerate);

	err=dl_mpg123_param(_handle, MPG123_RVA,
		SETTINGS->replayGain()?(SETTINGS->replayGainMode()?MPG123_RVA_ALBUM:MPG123_RVA_MIX):MPG123_RVA_OFF,
		0.);

	mpg123_id3v1* id3v1;
	mpg123_id3v2* id3v2;
	err=dl_mpg123_id3(_handle, &id3v1, &id3v2);
	if (err==MPG123_OK) {
		FXString info;

		if (id3v2 && id3v2->title)
			info=FXString(id3v2->title->p, id3v2->title->fill);
		else if (id3v1)
			info=FXString(id3v1->title, FXMIN(strlen(id3v1->title), 30)).trim();
		else
			info=FXString::null;
		FXTRACE((10, "TITLE: %s\n", info.text()));
		_metaData.addInfo(metaData::TITLE, info.text());

		if (id3v2 && id3v2->artist)
			info=FXString(id3v2->artist->p, id3v2->artist->fill);
		else if (id3v1)
			info=FXString(id3v1->artist, FXMIN(strlen(id3v1->artist), 30)).trim();
		else
			info=FXString::null;
		FXTRACE((10, "ARTIST: %s\n", info.text()));
		_metaData.addInfo(metaData::ARTIST, info.text());

		if (id3v2 && id3v2->album)
			info=FXString(id3v2->album->p, id3v2->album->fill);
		else if (id3v1)
			info=FXString(id3v1->album, FXMIN(strlen(id3v1->album), 30)).trim();
		else
			info=FXString::null;
		FXTRACE((10, "ALBUM: %s\n", info.text()));
		_metaData.addInfo(metaData::ALBUM, info.text());

		if (id3v2 && id3v2->genre) {
			info=FXString(id3v2->genre->p, id3v2->genre->fill);
			if (info.find('(')>=0) {
				const FXuint idx=FXUIntVal(info.after('(').before(')'));
				if (idx<GENRES)
					info=FXString(genre[idx]);
			}
		} else if (id3v1 && id3v1->genre<GENRES)
			info=genre[id3v1->genre];
		else
			info=FXString::null;
		FXTRACE((10, "GENRE: %s\n", info.text()));
		_metaData.addInfo(metaData::GENRE, info.text());

		if (id3v2 && id3v2->year)
			info=FXString(id3v2->year->p, id3v2->year->fill);
		else if (id3v1)
			info=FXString(id3v1->year, FXMIN(strlen(id3v1->year), 4)).trim();
		else
			info=FXString::null;
		FXTRACE((10, "YEAR: %s\n", info.text()));
		_metaData.addInfo(metaData::YEAR, info.text());

		if (id3v1 && id3v1->comment[28]==0)
#if FOX_MINOR>6
			info=FXString::value(id3v1->comment[29]);
#else
			info=FXStringVal(id3v1->comment[29]);
#endif
		else
			info=FXString::null;
		FXTRACE((10, "TRACK: %s\n", info.text()));
		_metaData.addInfo(metaData::TRACK, info.text());

		info=FXString::null;
		if (id3v2) {
			for (unsigned int i=0; i<id3v2->comments; i++) {
				mpg123_text *entry = &id3v2->comment_list[i];
				if (entry->description.fill==0 || entry->description.p[0]==0)
					info=FXString(entry->text.p, entry->text.fill).trim();
			}
		}

		if (info.empty() && id3v1)
			info=FXString(id3v1->comment, FXMIN(strlen(id3v1->comment), 30)).trim();

		if (!info.empty()) {
			FXTRACE((10, "COMMENT: %s\n", info.text()));
			_metaData.addInfo(metaData::COMMENT, info.text());
		}
	}

	_count++;

	songLoaded(fileName.text());
	return true;

error:
	fxmessage(dl_mpg123_plain_strerror(err));
	_file[_fd].close();
	return false;
}

void mpg123Backend::close() {

	if (!songLoaded())
		return;

	dl_mpg123_close(_handle);
	dl_mpg123_delete(_handle);

	_file[_fd].close();

	songLoaded(0);
}

bool mpg123Backend::rewind() {

	if (songLoaded()) {
		dl_mpg123_seek(_handle, 0, SEEK_SET);
		return true;
	}

	return false;
}

ssize_t mpg123Backend::read_func(int fd, void* ptr, size_t size) {

	return _file[fd].readBlock((char*)ptr, size);
}

off_t mpg123Backend::seek_func(int fd, off_t offset, int whence) {

	FXuint from;
	switch (whence) {
	case SEEK_SET:
		from=FXIO::Begin;
		break;
	case SEEK_CUR:
		from=FXIO::Current;
		break;
	case SEEK_END:
		from=FXIO::End;
		break;
	default:
		return -1;
	}
	return _file[fd].position(offset, from);
}

/*****************************************************************/

#define MPGSETTINGS	mpg123Backend::_settings

FXDEFMAP(mpg123Config) mpg123ConfigMap[] = {
	// Message_Type		ID				Message_Handler
	FXMAPFUNC(SEL_COMMAND,	mpg123Config::ID_DECODER,		mpg123Config::onCmdDecoder),
};

FXIMPLEMENT(mpg123Config, configFrame, mpg123ConfigMap, ARRAYNUMBER(mpg123ConfigMap))

mpg123Config::mpg123Config(FXPacker* win) :
	configFrame(win, mpg123Backend::name, CREDITS, LINK) {

	_fastscan.connect(MPGSETTINGS.fastscan);

	new FXLabel(matrix(), tr("Decoder"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
	FXListBox *decBox=new FXListBox(matrix(), this, mpg123Config::ID_DECODER, LAYOUT_FILL|FRAME_NORMAL);
	decBox->fillItems(mpg123Backend::_decoders);
	const FXint numItems=decBox->getNumItems();
	decBox->setNumVisible(FXMIN(numItems, 5));
	const FXint curItem=decBox->findItem(MPGSETTINGS.decoder);
	if (curItem>=0)
		decBox->setCurrentItem(curItem);

	new FXVerticalSeparator(extraLeft());

	FXVerticalFrame *vert=new FXVerticalFrame(extraLeft());
	new FXCheckButton(vert, tr("Fast scan\tScan only few frame of file, time detection may be inaccurate"),
		&_fastscan, FXDataTarget::ID_VALUE);
}

long mpg123Config::onCmdDecoder(FXObject*, FXSelector, void* ptr) {

	const FXint val=(FXint)(FXival)ptr;
	MPGSETTINGS.decoder=mpg123Backend::_decoders.section('\n', val);
	return 1;
}


#endif
